#[derive(Hash, PartialEq, Eq, Clone, Debug, Copy)]
pub struct Point(pub i32, pub i32);

impl Point {
    pub fn get_neighbors(&self) -> Vec<Point> {
        vec![self.left(), self.right(), self.up(), self.down()]
    }

    pub fn left(&self) -> Point {
        Point(self.0 - 1, self.1)
    }

    pub fn right(&self) -> Point {
        Point(self.0 + 1, self.1)
    }

    // y values are reversed in ncurses
    pub fn down(&self) -> Point {
        Point(self.0, self.1 + 1)
    }

    pub fn up(&self) -> Point {
        Point(self.0, self.1 - 1)
    }
}

#[derive(Clone, Copy, PartialEq, Eq, Debug)]
pub enum Direction {
    Up = 0,
    Right = 1,
    Down = 2,
    Left = 3,
}

impl Direction {
    pub fn from_u8(v: u8) -> Direction {
        match v {
            0 => Direction::Up,
            1 => Direction::Right,
            2 => Direction::Down,
            3 => Direction::Left,
            _ => panic!("Shouldn't happen")
        }
    }

    pub fn cw(&self) -> Direction {
        let val = *self as u8;
        Direction::from_u8((val + 1) % 4)
    }

    pub fn ccw(&self) -> Direction {
        let val = *self as u8;
        if val == 0 {
            Direction::Left
        } else {
            Direction::from_u8(val - 1)
        }
    }

    pub fn relative_point(&self, p: &Point) -> Point {
        match self {
            Direction::Up => p.up(),
            Direction::Left => p.left(),
            Direction::Right => p.right(),
            Direction::Down => p.down(),
        }
    }
}
