use crate::lib::point::Point;
use ncurses::*;

pub struct Screen {
    pub center: Point,
    pub max_x: i32,
    pub max_y: i32,
}

pub fn init_screen() -> Screen {
    initscr();

    start_color();
    init_pair(0, 0, 1);
    /* Invisible cursor. */
    curs_set(CURSOR_VISIBILITY::CURSOR_INVISIBLE);

    let mut max_x = 0;
    let mut max_y = 0;

    getmaxyx(stdscr(), &mut max_y, &mut max_x);
    
    Screen::new(max_x, max_y)
}

impl Screen {
    pub fn new(max_x: i32, max_y: i32) -> Screen {
        Screen {
            center: Point(max_x / 2, max_y / 2),
            max_x,
            max_y,
        }
    }

    pub fn is_in_bounds(&self, pos: &Point) -> bool {
        (pos.0 > 0 && pos.0 < self.max_x) && (pos.1 > 0 && pos.1 < self.max_y)
    }

    pub fn render(&self, p: &Point, char: &str) {
        mvprintw(p.1, p.0, char);
    }

    pub fn get_dimensions(&self) -> (Point, Point) {
        (Point(0,0), Point(self.max_x, self.max_y))    
    }
}

pub enum BoardCommand {
    LayEgg(Point, u32),
    SpawnFood(Point),
    Hatch(u32, u32),
    SpawnAnt,
    Noop,
}
